@@ -6,7 +6,7 @@ use JSON qw(from_json to_json);
 use Lemonldap::NG::Portal::Main::Constants qw(
 );
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.8';
 
 extends 'Lemonldap::NG::Portal::Main::SecondFactor';
 
@@ -65,7 +65,7 @@ sub run {
 
     my %tplPrms = (
         MAIN_LOGO   => $self->conf->{portalMainLogo},
-        SKIN        => $self->conf->{portalSkin},
+        SKIN        => $self->p->getSkin($req),
         TOKEN       => $token,
         CHECKLOGINS => $checkLogins
     );
@@ -86,12 +86,10 @@ sub run {
             }
 
             # Get registered keys
-            my @rk;
-            foreach ( @{ $req->data->{crypter} } ) {
-                push @rk,
-                  { keyHandle => $_->{keyHandle}, version => $data->{version} };
-
-            }
+            my @rk =
+              map {
+                { keyHandle => $_->{keyHandle}, version => $data->{version} }
+              } @{ $req->data->{crypter} };
 
             $self->ott->updateToken( $token, __ch => $data->{challenge} );
 
@@ -120,9 +118,7 @@ sub verify {
     if ( $req->param('signature') ) {
         $self->logger->debug('UTOTP: U2F response detected');
         my $r1 = $self->u2f->verify( $req, $session );
-        if ( $r1 == PE_OK ) {
-            return PE_OK;
-        }
+        return PE_OK if ( $r1 == PE_OK );
     }
     if ( $req->param('code') ) {
         $self->logger->debug('UTOTP: TOTP response detected');
