@@ -18,7 +18,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
 );
 use String::Random qw/random_string/;
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.9';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::OpenIDConnect',
@@ -321,6 +321,9 @@ sub run {
                 );
             }
 
+            my $spAuthnLevel = $self->conf->{oidcRPMetaDataOptions}->{$rp}
+              ->{oidcRPMetaDataOptionsAuthnLevel} || 0;
+
             # Check if user needs to be reauthenticated
             my $prompt = $oidc_request->{'prompt'};
             if (
@@ -334,6 +337,7 @@ sub run {
                 $self->logger->debug(
 "Reauthentication required by Relying Party in prompt parameter"
                 );
+                $req->pdata->{targetAuthnLevel} = $spAuthnLevel;
                 return $self->reAuth($req);
             }
 
@@ -343,9 +347,23 @@ sub run {
                 $self->logger->debug(
 "Reauthentication forced because authentication time ($_lastAuthnUTime) is too old (>$max_age s)"
                 );
+                $req->pdata->{targetAuthnLevel} = $spAuthnLevel;
                 return $self->reAuth($req);
             }
 
+            # Check if we have sufficient auth level
+            my $authenticationLevel =
+              $req->{sessionInfo}->{authenticationLevel} || 0;
+            if ( $authenticationLevel < $spAuthnLevel ) {
+                $self->logger->debug(
+                        "Insufficient authentication level for service $rp"
+                      . " (has: $authenticationLevel, want: $spAuthnLevel)" );
+
+                # Reauth with sp auth level as target
+                $req->pdata->{targetAuthnLevel} = $spAuthnLevel;
+                return $self->upgradeAuth($req);
+            }
+
             # Check scope validity
             # We use a slightly more relaxed version of
             # https://tools.ietf.org/html/rfc6749#appendix-A.4
@@ -393,11 +411,8 @@ sub run {
 
                 # Check that id_token_hint sub match current user
                 my $sub = $self->getIDTokenSub($id_token_hint);
-                my $user_id_attribute =
-                  $self->conf->{oidcRPMetaDataOptions}->{$rp}
-                  ->{oidcRPMetaDataOptionsUserIDAttr}
-                  || $self->conf->{whatToTrace};
-                my $user_id = $req->{sessionInfo}->{$user_id_attribute};
+                my $user_id =
+                  $self->getUserIDForRP( $req, $rp, $req->{sessionInfo} );
                 unless ( $sub eq $user_id ) {
                     $self->userLogger->error(
                         "ID Token hint sub $sub does not match user $user_id");
@@ -707,10 +722,7 @@ sub run {
                     $session_state
                 );
 
-                $self->logger->debug("Redirect user to $response_url");
-                $req->urldc($response_url);
-
-                return PE_REDIRECT;
+                return $self->_redirectToUrl( $req, $response_url );
             }
 
             # Implicit Flow
@@ -780,10 +792,7 @@ sub run {
                     $session_state
                 );
 
-                $self->logger->debug("Redirect user to $response_url");
-                $req->urldc($response_url);
-
-                return PE_REDIRECT;
+                return $self->_redirectToUrl( $req, $response_url );
             }
 
             # Hybrid Flow
@@ -885,9 +894,7 @@ sub run {
                     $session_state
                 );
 
-                $self->logger->debug("Redirect user to $response_url");
-                $req->urldc($response_url);
-                return PE_REDIRECT;
+                return $self->_redirectToUrl( $req, $response_url );
             }
 
             $self->logger->debug("None flow has been selected");
@@ -969,9 +976,7 @@ sub run {
                       $self->buildLogoutResponse( $post_logout_redirect_uri,
                         $state );
 
-                    $self->logger->debug("Redirect user to $response_url");
-                    $req->urldc($response_url);
-                    return PE_REDIRECT;
+                    return $self->_redirectToUrl( $req, $response_url );
                 }
                 return $req->param('confirm') == 1
                   ? ( $err ? $err : PE_LOGOUT_OK )
@@ -1090,11 +1095,7 @@ sub _handlePasswordGrant {
         }
     }
 
-    my $user_id_attribute =
-      $self->conf->{oidcRPMetaDataOptions}->{$rp}
-      ->{oidcRPMetaDataOptionsUserIDAttr}
-      || $self->conf->{whatToTrace};
-    my $user_id = $req->sessionInfo->{$user_id_attribute};
+    my $user_id = $self->getUserIDForRP( $req, $rp, $req->sessionInfo );
 
     $self->logger->debug("Found corresponding user: $user_id");
 
@@ -1228,11 +1229,7 @@ sub _handleAuthorizationCodeGrant {
         return $self->p->sendError( $req, 'invalid_grant', 400 );
     }
 
-    my $user_id_attribute =
-      $self->conf->{oidcRPMetaDataOptions}->{$rp}
-      ->{oidcRPMetaDataOptionsUserIDAttr}
-      || $self->conf->{whatToTrace};
-    my $user_id = $apacheSession->data->{$user_id_attribute};
+    my $user_id = $self->getUserIDForRP( $req, $rp, $apacheSession->data );
 
     $self->logger->debug("Found corresponding user: $user_id");
 
@@ -1448,11 +1445,7 @@ sub _handleRefreshTokenGrant {
                 'Invalid request', 401 );
         }
 
-        my $user_id_attribute =
-          $self->conf->{oidcRPMetaDataOptions}->{$rp}
-          ->{oidcRPMetaDataOptionsUserIDAttr}
-          || $self->conf->{whatToTrace};
-        $user_id = $session->data->{$user_id_attribute};
+        $user_id = $self->getUserIDForRP( $req, $rp, $session->data );
 
         $auth_time = $session->data->{_lastAuthnUTime};
 
@@ -1518,11 +1511,7 @@ sub _handleRefreshTokenGrant {
             $refreshSession->data->{$_} = $req->sessionInfo->{$_};
         }
 
-        my $user_id_attribute =
-          $self->conf->{oidcRPMetaDataOptions}->{$rp}
-          ->{oidcRPMetaDataOptionsUserIDAttr}
-          || $self->conf->{whatToTrace};
-        $user_id = $req->sessionInfo->{$user_id_attribute};
+        $user_id = $self->getUserIDForRP( $req, $rp, $req->sessionInfo );
         $self->logger->debug("Found corresponding user: $user_id");
 
         $auth_time = $refreshSession->data->{auth_time};
@@ -1758,11 +1747,8 @@ sub introspection {
 
         # The ID attribute we choose is the one of the calling webservice,
         # which might be different from the OIDC client the token was issued to.
-            my $user_id_attribute =
-              $self->conf->{oidcRPMetaDataOptions}->{$rp}
-              ->{oidcRPMetaDataOptionsUserIDAttr}
-              || $self->conf->{whatToTrace};
-            $response->{sub}   = $apacheSession->data->{$user_id_attribute};
+            $response->{sub} =
+              $self->getUserIDForRP( $req, $rp, $apacheSession->data );
             $response->{scope} = $oidcSession->{data}->{scope}
               if $oidcSession->{data}->{scope};
             $response->{client_id} =
@@ -2008,11 +1994,8 @@ sub logout {
                 if ( $rpConf->{oidcRPMetaDataOptionsLogoutType} eq 'front' ) {
                     if ( $rpConf->{oidcRPMetaDataOptionsLogoutSessionRequired} )
                     {
-                        my $user_id_attribute =
-                          $self->conf->{oidcRPMetaDataOptions}->{$rp}
-                          ->{oidcRPMetaDataOptionsUserIDAttr}
-                          || $self->conf->{whatToTrace};
-                        my $user_id = $req->{sessionInfo}->{$user_id_attribute};
+                        my $user_id = $self->getUserIDForRP( $req, $rp,
+                            $req->{sessionInfo} );
                         $url .= ( $url =~ /\?/ ? '&' : '?' )
                           . build_urlencoded(
                             iss => $self->iss,
@@ -2074,6 +2057,15 @@ sub metadata {
         push( @$grant_types, "password" );
     }
 
+    # If one of the RPs has refresh tokens enabled
+    if (
+        grep { $self->oidcRPList->{$_}->{oidcRPMetaDataOptionsRefreshToken} }
+        keys %{ $self->oidcRPList }
+      )
+    {
+        push( @$grant_types, "refresh_token" );
+    }
+
     if ( $self->conf->{oidcServiceAllowHybridFlow} ) {
         push( @$response_types,
             "code id_token",
@@ -2188,6 +2180,12 @@ sub exportRequestParameters {
     if ( $req->param('client_id') ) {
         my $rp = $self->getRP( $req->param('client_id') );
         $req->env->{"llng_oidc_rp"} = $rp if $rp;
+
+        # Store target authentication level in pdata
+        my $targetAuthnLevel = $self->conf->{oidcRPMetaDataOptions}->{$rp}
+          ->{oidcRPMetaDataOptionsAuthnLevel};
+        $req->pdata->{targetAuthnLevel} = $targetAuthnLevel
+          if $targetAuthnLevel;
     }
 
     return PE_OK;
@@ -2278,11 +2276,7 @@ sub _generateIDToken {
         }
     }
 
-    my $user_id_attribute =
-      $self->conf->{oidcRPMetaDataOptions}->{$rp}
-      ->{oidcRPMetaDataOptionsUserIDAttr}
-      || $self->conf->{whatToTrace};
-    my $user_id = $req->{sessionInfo}->{$user_id_attribute};
+    my $user_id = $self->getUserIDForRP( $req, $rp, $req->{sessionInfo} );
 
     my $id_token_payload_hash = {
         iss       => $self->iss,                            # Issuer Identifier
@@ -2323,4 +2317,15 @@ sub _generateIDToken {
     return $self->createIDToken( $id_token_payload_hash, $rp );
 }
 
+sub _redirectToUrl {
+    my ( $self, $req, $response_url ) = @_;
+
+    # We must clear hidden form fields saved from the request (#2085)
+    $self->p->clearHiddenFormValue($req);
+    $self->logger->debug("Redirect user to $response_url");
+    $req->urldc($response_url);
+
+    return PE_REDIRECT;
+}
+
 1;
