@@ -8,7 +8,7 @@ use Lemonldap::NG::Portal::Main::Constants
 
 extends 'Lemonldap::NG::Common::Module';
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.9';
 
 # PROPERTIES
 
@@ -113,7 +113,15 @@ sub init {
 
 sub getUser {
     my ( $self, $req, %args ) = @_;
-    return PE_LDAPCONNECTFAILED unless $self->ldap and $self->bind();
+
+    $self->validateLdap;
+
+    unless ( $self->ldap ) {
+        return PE_LDAPCONNECTFAILED;
+    }
+
+    $self->bind();
+
     my $mesg = $self->ldap->search(
         base   => $self->conf->{ldapBase},
         scope  => 'sub',
@@ -126,7 +134,8 @@ sub getUser {
         attrs => $self->attrs,
     );
     if ( $mesg->code() != 0 ) {
-        $self->logger->error( 'LDAP Search error: ' . $mesg->error );
+        $self->logger->error(
+            'LDAP Search error ' . $mesg->code . ": " . $mesg->error );
         return PE_LDAPERROR;
     }
     if ( $mesg->count() > 1 ) {
@@ -135,7 +144,7 @@ sub getUser {
         return PE_BADCREDENTIALS;
     }
     unless ( $req->data->{ldapentry} = $mesg->entry(0) ) {
-        $self->userLogger->warn("$req->{user} was not found in LDAP directory");
+        $self->userLogger->warn("$req->{user} was not found in LDAP directory (".$req->address.")");
         eval { $self->p->_authentication->setSecurity($req) };
         return PE_BADCREDENTIALS;
     }
@@ -143,15 +152,23 @@ sub getUser {
     PE_OK;
 }
 
-# Test LDAP connection before trying to bind
-sub bind {
-    my $self = shift;
+# Validate LDAP connection before use
+sub validateLdap {
+    my ($self) = @_;
     unless ($self->ldap
         and $self->ldap->root_dse( attrs => ['supportedLDAPVersion'] ) )
     {
         $self->ldap->DESTROY if ( $self->ldap );
         $self->ldap( $self->newLdap );
     }
+}
+
+# Bind
+sub bind {
+    my $self = shift;
+
+    $self->validateLdap;
+
     return undef unless ( $self->ldap );
     my $msg = $self->ldap->bind(@_);
     if ( $msg->code ) {
