@@ -11,10 +11,11 @@ use JSON qw(from_json);
 use MIME::Entity;
 use Email::Sender::Simple qw(sendmail);
 use Email::Sender::Transport::SMTP qw();
+use Email::Date::Format qw(email_date);
 use MIME::Base64;
 use Encode;
 
-our $VERSION = '2.0.4';
+our $VERSION = '2.0.9';
 
 our $transport;
 
@@ -81,15 +82,30 @@ has transport => (
     },
 );
 
+sub loadMailTemplate {
+    my ( $self, $req, $name, %prm ) = @_;
+
+    # HTML::Template cache interferes with email translation (#1897)
+    $prm{cache} = 0 unless defined $prm{cache};
+    $prm{params}->{STATIC_PREFIX} = $self->p->staticPrefix;
+    my %extra =
+        $self->p->can('tplParams')
+      ? $self->p->tplParams($req)
+      : ();
+    $prm{params}->{$_} = $extra{$_} for keys %extra;
+
+    return $self->loadTemplate( $req, $name, %prm );
+}
+
 sub translate {
     my ( $self, $req ) = @_;
 
     # Get language using llnglanguage cookie
     my $lang_code = $req->cookies->{llnglanguage} || 'en';
-    my $json = $self->conf->{templateDir} . "/common/mail/$lang_code.json";
+    my $json      = $self->conf->{templateDir} . "/common/mail/$lang_code.json";
     $json = $self->conf->{templateDir} . '/common/mail/en.json'
       unless ( -f $json );
-    open F, $json
+    open F, '<', $json
       or die 'Installation error: '
       . $!
       . " ($self->{conf}->{templateDir}/$lang_code.json or $self->{conf}->{templateDir}/common/mail/en.json)";
@@ -100,7 +116,7 @@ sub translate {
 
     if ($langOver) {
         for my $k ( keys %{ $langOver->{all} || {} } ) {
-            $lang->{$k} = $langOver->{$lang_code}->{$k};
+            $lang->{$k} = $langOver->{all}->{$k};
         }
         for my $k ( keys %{ $langOver->{$lang_code} || {} } ) {
             $lang->{$k} = $langOver->{$lang_code}->{$k};
@@ -128,7 +144,7 @@ sub gen_password {
 # @return boolean result
 sub send_mail {
     my ( $self, $mail, $subject, $body, $html ) = @_;
-    $self->logger->debug("send_mail called to send \"$subject\" to $mail");
+    $self->logger->info("send_mail called to send \"$subject\" to $mail");
 
     # Encode the body with the given charset
     $body    = encode( $self->charset, decode( 'utf-8', $body ) );
@@ -171,6 +187,7 @@ sub send_mail {
                 ),
                 Subject => $subject,
                 Type    => 'multipart/related',
+                Date    => email_date,
             );
 
             # Attach HTML message
@@ -202,6 +219,7 @@ sub send_mail {
                 Data       => $body,
                 Type       => 'text/plain',
                 Charset    => $self->charset,
+                Date       => email_date,
             );
         }
 
