@@ -8,13 +8,14 @@
 #                  of lemonldap-ng.ini) and underlying handler configuration
 package Lemonldap::NG::Portal::Main::Init;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.9';
 
 package Lemonldap::NG::Portal::Main;
 
 use strict;
 use Mouse;
 use Regexp::Assemble;
+use Lemonldap::NG::Common::Util qw(getSameSite);
 
 # PROPERTIES
 
@@ -87,6 +88,9 @@ has csp => ( is => 'rw' );
 # Cross-Origine Resource Sharing headers
 has cors => ( is => 'rw' );
 
+# Cookie SameSite value
+has cookieSameSite => ( is => 'rw' );
+
 # INITIALIZATION
 
 sub init {
@@ -123,13 +127,30 @@ sub init {
     Lemonldap::NG::Handler::Main->onReload( $self, 'reloadConf' );
 
     # Handler::PSGI::Try initialization
-    return 0 unless ( $self->SUPER::init( $self->localConfig ) );
+    unless ( $self->SUPER::init( $self->localConfig ) ) {
+        $self->logger->error( 'Initialization failed: ' . $self->error );
+        $self->error(
+"Initialization failed! Enable debug logs, reload your web server and catch main error..."
+        );
+        return 0;
+    }
     if ( $self->error ) {
         $self->logger->error( $self->error );
         return 0;
     }
 
-    # Handle requests (other path may be declared in enabled plugins)
+    # Default routes must point to routines declared above
+    $self->defaultAuthRoute('');
+    $self->defaultUnauthRoute('');
+    return 1;
+}
+
+sub setPortalRoutes {
+    my ($self) = @_;
+    $self->authRoutes(
+        { GET => {}, POST => {}, PUT => {}, DELETE => {}, OPTIONS => {} } );
+    $self->unAuthRoutes(
+        { GET => {}, POST => {}, PUT => {}, DELETE => {}, OPTIONS => {} } );
     $self
 
       # "/" or undeclared paths
@@ -167,11 +188,15 @@ sub init {
     $self->defaultAuthRoute('');
     $self->defaultUnauthRoute('');
     return 1;
+
 }
 
 sub reloadConf {
     my ( $self, $conf ) = @_;
 
+    # Handle requests (other path may be declared in enabled plugins)
+    $self->setPortalRoutes;
+
     # Reinitialize $self->conf
     %{ $self->{conf} } = %{ $self->localConfig };
 
@@ -203,9 +228,11 @@ sub reloadConf {
     {
         my $header = $_;
         my $prm    = $self->conf->{ 'cors' . $_ };
-        $header =~ s/_/-/;
-        $prm    =~ s/\s+//;
-        $cors .= "Access-Control-$header;$prm;";
+        if ( $header and $prm ) {
+            $header =~ s/_/-/;
+            $prm    =~ s/\s+//;
+            $cors .= "Access-Control-$header;$prm;";
+        }
     }
     $self->cors($cors);
     $self->logger->debug( "Initialized CORS headers : " . $self->cors );
@@ -245,6 +272,11 @@ sub reloadConf {
     }
     $self->conf->{domain} =~ s/^([^\.])/.$1/;
 
+    # Initialize cookie SameSite value
+    $self->cookieSameSite( getSameSite( $self->conf ) );
+    $self->logger->debug(
+        "Cookies will use SameSite=" . $self->cookieSameSite );
+
     # Load menu
     # ---------
     $self->menu( $self->loadPlugin('::Main::Menu') );
@@ -356,11 +388,19 @@ sub reloadConf {
 
     # Clean $req->pdata after authentication
     push @{ $self->endAuth }, sub {
-        unless ( $_[0]->pdata->{keepPdata} ) {
-            $self->logger->debug('Cleaning pdata');
-            $_[0]->pdata( {} );
-            $self->userLogger->notice( $_[0]->user . ' connected' )
-              if $_[0]->user;
+        my $tmp = $_[0]->pdata->{keepPdata} //= [];
+        foreach my $k ( keys %{ $_[0]->pdata } ) {
+            unless ( grep { $_ eq $k } @$tmp ) {
+                $self->logger->debug("Removing $k from pdata");
+                delete $_[0]->pdata->{$k};
+            }
+        }
+        my $user_log = $_[0]->{sessionInfo}->{ $self->conf->{whatToTrace} };
+        $self->userLogger->notice( $user_log . ' connected' ) if $user_log;
+        if (@$tmp) {
+            $self->logger->debug(
+                'Add ' . join( ',', @$tmp ) . ' in keepPdata' );
+            $_[0]->pdata->{keepPdata} = $tmp;
         }
         return PE_OK;
     };
@@ -504,4 +544,25 @@ sub displayError {
         'Portal error, contact your administrator', 500 );
 }
 
+# This helper method builds a rule from a string expression
+# - $rule: rule text
+# - $ruleDesc optional hint of what the rule is for, to display in error message
+# returns undef if the rule syntax was invalid
+sub buildRule {
+    my ( $self, $rule, $ruleDesc ) = @_;
+    if ($ruleDesc) {
+        $ruleDesc = " $ruleDesc ";
+    }
+    else {
+        $ruleDesc = " ";
+    }
+    my $compiledRule =
+      $self->HANDLER->buildSub( $self->HANDLER->substitute($rule) );
+    unless ($compiledRule) {
+        my $error = $self->HANDLER->tsv->{jail}->error || '???';
+        $self->logger->error( "Bad" . $ruleDesc . "rule: " . $error );
+    }
+    return $compiledRule,;
+}
+
 1;
