@@ -12,30 +12,31 @@ use Mouse;
 use MIME::Base64;
 use IO::String;
 use URI::Escape;
+use URI;
 use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
   PE_RENEWSESSION
+  PE_UPGRADESESSION
 );
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.9';
 
 # PROPERTIES
 
-has type => ( is => 'rw' );
-
-has path => ( is => 'rw' );
-
+has type  => ( is => 'rw' );
+has path  => ( is => 'rw' );
 has ipath => ( is => 'rw' );
-
-has _ott => (
+has _ott  => (
     is      => 'rw',
     lazy    => 1,
     default => sub {
-        my $ott = $_[0]->{p}->loadModule('::Lib::OneTimeToken');
-        $ott->timeout( $_[0]->{conf}->{formTimeout} );
+        my $ott     = $_[0]->{p}->loadModule('::Lib::OneTimeToken');
+        my $timeout = $_[0]->{conf}->{issuersTimeout}
+          // $_[0]->{conf}->{formTimeout};
+        $ott->timeout($timeout);
         return $ott;
     }
 );
@@ -50,6 +51,12 @@ sub beforeLogout { 'logout' }
 
 sub init {
     my ($self) = @_;
+    if ( $self->conf->{forceGlobalStorageIssuerOTT} ) {
+        $self->logger->debug(
+            "-> Issuer tokens will be stored into global storage");
+        $self->_ott->cache(undef);
+    }
+
     my $type = ref( $_[0] );
     $type =~ s/.*:://;
     $self->type($type);
@@ -77,6 +84,11 @@ sub init {
 
 sub _redirect {
     my ( $self, $req, @path ) = @_;
+    if (    $req->pdata->{issuerTs}
+        and $req->pdata->{issuerTs} + $self->conf->{issuersTimeout} < time )
+    {
+        $self->cleanAllPdata($req);
+    }
     my $restore;
     my $ir;
     unless ( $self->can('ssoMatch') and not $self->ssoMatch($req) ) {
@@ -85,8 +97,12 @@ sub _redirect {
         $self->logger->debug('Processing _redirect');
         $ir = $req->pdata->{ $self->ipath } ||= $self->storeRequest($req);
         $req->pdata->{ $self->ipath . 'Path' } = \@path;
-        $req->pdata->{keepPdata} = 1;
+        $self->logger->debug(
+            'Add ' . $self->ipath . ', ' . $self->ipath . 'Path in keepPdata' );
+        push @{ $req->pdata->{keepPdata} }, $self->ipath, $self->ipath . 'Path';
         $req->{urldc} = $self->conf->{portal} . '/' . $self->path;
+        $req->pdata->{_url}     = encode_base64( $req->urldc, '' );
+        $req->pdata->{issuerTs} = time;
     }
     else {
         $self->logger->debug('Not seen as Issuer request, skipping');
@@ -110,8 +126,7 @@ sub _redirect {
 
                     # Restore urldc if auth doesn't need to dial with browser
                     $self->restoreRequest( $req, $ir );
-                    delete $req->pdata->{ $self->ipath };
-                    delete $req->pdata->{ $self->ipath . 'Path' };
+                    $self->cleanPdata($req);
                     return $self->run( @_, @path );
                 }
                 : ()
@@ -124,18 +139,28 @@ sub _redirect {
 sub _forAuthUser {
     my ( $self, $req, @path ) = @_;
 
+    if (    $req->pdata->{issuerTs}
+        and $req->pdata->{issuerTs} + $self->conf->{issuersTimeout} < time )
+    {
+        $self->cleanAllPdata($req);
+    }
     $self->logger->debug('Processing _forAuthUser');
     if ( my $r = $req->pdata->{ $self->ipath } ) {
         $self->logger->debug("Restoring request to $self->{path} issuer");
         $self->restoreRequest( $req, $r );
         @path = @{ $req->pdata->{ $self->ipath . 'Path' } }
           if ( $req->pdata->{ $self->ipath . 'Path' } );
+
+        # In case a confirm form is shown, we need it to POST on the
+        # current Path
+        $req->data->{confirmFormAction} = URI->new( $req->uri )->path;
     }
 
     # Clean pdata: keepPdata has been set, so pdata must be cleaned here
     $self->logger->debug('Cleaning pdata');
-    $req->pdata( {} );
-    $req->urlNotBase64(1) if ( ref($self) =~ /::CAS$/ );
+    $self->cleanPdata($req);
+
+    $req->maybeNotBase64(1) if ( ref($self) =~ /::CAS$/ );
     $req->mustRedirect(1);
     return $self->p->do(
         $req,
@@ -150,6 +175,37 @@ sub _forAuthUser {
     );
 }
 
+sub cleanAllPdata {
+    my ( $self, $req ) = @_;
+    foreach my $k ( keys %{ $req->pdata } ) {
+        if ( exists $req->pdata->{ $k . 'Path' } ) {
+            $self->cleanPdata( $req, $k );
+        }
+    }
+}
+
+sub cleanPdata {
+    my ( $self, $req, $path ) = @_;
+    $path ||= $self->ipath;
+    for my $s ( $path, $path . 'Path' ) {
+        if ( $req->pdata->{$s} ) {
+            $self->logger->debug("Removing $s key from pdata");
+            delete $req->pdata->{$s};
+        }
+    }
+    if ( $req->pdata->{keepPdata} and ref $req->pdata->{keepPdata} ) {
+        @{ $req->pdata->{keepPdata} } =
+          grep {
+                  $_ ne $path
+              and $_ ne $path . 'Path'
+              ? 1
+              : ( $self->logger->debug("Removing $_ from keepPdata") and 0 )
+          } @{ $req->pdata->{keepPdata} };
+        delete $req->pdata->{keepPdata}
+          unless ( @{ $req->pdata->{keepPdata} } );
+    }
+}
+
 sub storeRequest {
     my ( $self, $req ) = @_;
     $self->logger->debug('Store issuer request');
@@ -190,10 +246,24 @@ qq'<script type="text/javascript" src="$self->{p}->{staticPrefix}/common/js/auto
     $req->data->{_url} =
       encode_base64( $self->conf->{portal} . $req->path_info, '' );
     $req->pdata->{ $self->ipath } = $self->storeRequest($req);
-    $req->pdata->{keepPdata} = 1;
+    push @{ $req->pdata->{keepPdata} }, $self->ipath, $self->ipath . 'Path';
+    $req->pdata->{issuerTs} = time;
     return PE_RENEWSESSION;
 }
 
+sub upgradeAuth {
+    my ( $self, $req ) = @_;
+    $req->data->{customScript} =
+qq'<script type="text/javascript" src="$self->{p}->{staticPrefix}/common/js/autoRenew.min.js"></script>'
+      if ( $self->conf->{skipUpgradeConfirmation} );
+    $req->data->{_url} =
+      encode_base64( $self->conf->{portal} . $req->path_info, '' );
+    $req->pdata->{ $self->ipath } = $self->storeRequest($req);
+    push @{ $req->pdata->{keepPdata} }, $self->ipath, $self->ipath . 'Path';
+    $req->pdata->{issuerTs} = time;
+    return PE_UPGRADESESSION;
+}
+
 1;
 __END__
 
