@@ -14,7 +14,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_IMPERSONATION_SERVICE_NOT_ALLOWED
 );
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.9';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Plugin
@@ -34,12 +34,13 @@ has ott => (
         return $ott;
     }
 );
-has rule   => ( is => 'rw', default => sub { 0 } );
-has idRule => ( is => 'rw', default => sub { 1 } );
+has rule                  => ( is => 'rw', default => sub { 0 } );
+has idRule                => ( is => 'rw', default => sub { 1 } );
+has unrestrictedUsersRule => ( is => 'rw', default => sub { 0 } );
 
 sub init {
     my ($self) = @_;
-    $self->addAuthRoute( switchcontext => 'run', ['POST'] )
+    $self->addAuthRoute( switchcontext => 'run',  ['POST'] )
       ->addAuthRoute( switchcontext => 'display', ['GET'] );
 
     # Parse ContextSwitching rules
@@ -59,6 +60,14 @@ sub init {
     );
     return 0 unless $self->idRule;
 
+    $self->unrestrictedUsersRule(
+        $self->p->buildRule(
+            $self->conf->{contextSwitchingUnrestrictedUsersRule},
+            'contextSwitchingUnrestrictedUsers'
+        )
+    );
+    return 0 unless $self->unrestrictedUsersRule;
+
     return 1;
 }
 
@@ -133,6 +142,7 @@ sub run {
     my $statut  = PE_OK;
     my $realId  = $req->{user};
     my $spoofId = $req->param('spoofId') || '';    # ContextSwitching required ?
+    my $unUser = $self->unrestrictedUsersRule->( $req, $req->userData ) || 0;
 
     # Check token
     if ( $self->ottRule->( $req, {} ) ) {
@@ -158,7 +168,7 @@ sub run {
 
     # ContextSwitching required -> Check user Id
     if ( $spoofId && $spoofId ne $req->{user} ) {
-        $self->logger->debug("Spoof Id: $spoofId");
+        $self->logger->debug("Spoofed Id: $spoofId");
         unless ( $spoofId =~ /$self->{conf}->{userControl}/o ) {
             $self->userLogger->warn('Malformed spoofed Id');
             $self->logger->debug(
@@ -173,7 +183,7 @@ sub run {
     }
 
     # Create spoofed session
-    $req = $self->_switchContext( $req, $spoofId );
+    $req = $self->_switchContext( $req, $spoofId, $unUser );
     $statut =
       ( $req->error == PE_BADCREDENTIALS ? PE_MALFORMEDUSER : $req->error )
       if $req->error;
@@ -181,13 +191,15 @@ sub run {
     # Main session
     $self->p->updateSession( $req, $req->sessionInfo );
     $self->userLogger->notice(
-        "ContextSwitching: Update $realId session with $spoofId session data");
+"ContextSwitching: Update \"$realId\" session with \"$spoofId\" session data"
+    );
 
+    $req->mustRedirect(1);
     return $self->p->do( $req, [ sub { $statut } ] );
 }
 
 sub _switchContext {
-    my ( $self, $req, $spoofId ) = @_;
+    my ( $self, $req, $spoofId, $unUser ) = @_;
     my $realSessionId = $req->userData->{_session_id};
     my $realAuthLevel = $req->userData->{authenticationLevel};
     my $realId        = $req->{user};
@@ -204,7 +216,7 @@ sub _switchContext {
     );
     if ( my $error = $self->p->process($req) ) {
         $self->userLogger->warn(
-                'ContextSwitching requested for an unvalid user ('
+                'ContextSwitching requested for an invalid user ('
               . $req->{user}
               . ")" )
           if ( $error == PE_BADCREDENTIALS );
@@ -213,10 +225,11 @@ sub _switchContext {
         $raz = 1;
     }
 
-    # Check identity rule if ContextSwitching required
-    unless ( $self->idRule->( $req, $req->sessionInfo ) ) {
+    # Check identities rule if ContextSwitching required
+    $self->logger->info("\"$realId\" is an unrestricted user!") if $unUser;
+    unless ( $unUser || $self->idRule->( $req, $req->sessionInfo ) ) {
         $self->userLogger->warn(
-                'ContextSwitching requested for an unvalid user ('
+                'ContextSwitching requested for an invalid user ('
               . $req->{user}
               . ")" );
         $self->logger->debug('Identity NOT authorized');
@@ -244,7 +257,7 @@ sub _switchContext {
     }
 
     $self->userLogger->notice(
-        "Start ContextSwitching: $realId becomes $spoofId ");
+        "Start ContextSwitching: \"$realId\" becomes \"$spoofId\"");
     return $req;
 }
 
@@ -261,7 +274,7 @@ sub _abortImpersonation {
 
     if ($abort) {
         $self->userLogger->notice(
-            "Abort ContextSwitching: $spoofId by $realId");
+            "Abort ContextSwitching: \"$spoofId\" by \"$realId\"");
         if ( my $abortSession = $self->p->getApacheSession( $req->id ) ) {
             $abortSession->remove;
         }
@@ -272,7 +285,7 @@ sub _abortImpersonation {
     }
     else {
         $self->userLogger->notice(
-            "Stop ContextSwitching for $realId with uid $spoofId");
+            "Stop ContextSwitching for \"$realId\" with uid \"$spoofId\"");
         $self->p->deleteSession($req);
     }
 
