@@ -2,19 +2,30 @@ package Lemonldap::NG::Portal::Plugins::Impersonation;
 
 use strict;
 use Mouse;
-use Lemonldap::NG::Portal::Main::Constants
-  qw( PE_OK PE_BADCREDENTIALS PE_IMPERSONATION_SERVICE_NOT_ALLOWED PE_MALFORMEDUSER );
+use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK PE_BADCREDENTIALS
+  PE_IMPERSONATION_SERVICE_NOT_ALLOWED
+  PE_MALFORMEDUSER
+);
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.9';
 
-extends 'Lemonldap::NG::Portal::Main::Plugin';
+extends 'Lemonldap::NG::Portal::Main::Plugin',
+  'Lemonldap::NG::Portal::Lib::_tokenRule';
 
 # INITIALIZATION
 
 use constant afterData => 'run';
 
-has rule   => ( is => 'rw', default => sub { 1 } );
-has idRule => ( is => 'rw', default => sub { 1 } );
+has rule                  => ( is => 'rw', default => sub { 1 } );
+has idRule                => ( is => 'rw', default => sub { 1 } );
+has unrestrictedUsersRule => ( is => 'rw', default => sub { 0 } );
+
+# Form timeout token generator (used if requireToken is set)
+has ott => ( is => 'rw' );
+
+# Captcha generator
+has captcha => ( is => 'rw' );
 
 sub hAttr {
     $_[0]->{conf}->{impersonationHiddenAttributes} . ' '
@@ -40,6 +51,25 @@ sub init {
     );
     return 0 unless $self->idRule;
 
+    $self->unrestrictedUsersRule(
+        $self->p->buildRule(
+            $self->conf->{impersonationUnrestrictedUsersRule},
+            'impersonationUnrestrictedUsers'
+        )
+    );
+    return 0 unless $self->unrestrictedUsersRule;
+
+    # Initialize Captcha if needed
+    if ( $self->{conf}->{captcha_login_enabled} ) {
+        $self->captcha( $self->p->loadModule('::Lib::Captcha') ) or return 0;
+    }
+
+    # Initialize form token if needed (captcha provides also a token)
+    else {
+        $self->ott( $self->p->loadModule('::Lib::OneTimeToken') ) or return 0;
+        $self->ott->timeout( $self->conf->{formTimeout} );
+    }
+
     return 1;
 }
 
@@ -50,15 +80,16 @@ sub run {
 
     return $req->authResult
       if $req->authResult >
-      PE_OK;    # Skip Impersonation if error during Auth process
+      PE_OK;    # Skip Impersonation if an error occurs during Auth process
 
     my $statut = PE_OK;
+    my $unUser = 0;
     my $loginHistory =
       $req->{sessionInfo}->{_loginHistory};    # Store login history
     $req->{user} ||= $req->{sessionInfo}->{_impUser};    # If 2FA is enabled
     my $spoofId = $req->param('spoofId')       # Impersonation required
       || $req->{sessionInfo}->{_impSpoofId}    # If 2FA is enabled
-      || $req->{user};                         # NO Impersonation required
+      || $req->{user};                         # Impersonation not required
 
     $self->logger->debug("No impersonation required")
       if ( $spoofId eq $req->{user} );
@@ -70,7 +101,7 @@ sub run {
         $statut  = PE_MALFORMEDUSER;
     }
 
-    # Check activation rule
+    # Check activation & unrestrictedUsers rules
     if ( $spoofId ne $req->{user} ) {
         $self->logger->debug("Spoof Id: $spoofId / Real Id: $req->{user}");
         unless ( $self->rule->( $req, $req->sessionInfo ) ) {
@@ -78,6 +109,7 @@ sub run {
             $spoofId = $req->{user};
             $statut  = PE_IMPERSONATION_SERVICE_NOT_ALLOWED;
         }
+        $unUser = $self->unrestrictedUsersRule->( $req, $req->sessionInfo );
     }
 
     # Fill spoof session
@@ -98,10 +130,11 @@ sub run {
         delete $req->{sessionInfo}->{$k};
     }
 
-    $spoofSession = $self->_userData( $req, $spoofId, $realSession );
+    $spoofSession = $self->_userData( $req, $spoofId, $realSession, $unUser );
     if ( $req->error ) {
+        $self->setSecurity($req);
         if ( $req->error == PE_BADCREDENTIALS ) {
-            $statut = PE_BADCREDENTIALS;
+            $statut = PE_BADCREDENTIALS;  # Catch error to preserve protected Id
         }
         else {
             return $req->error;
@@ -126,19 +159,20 @@ sub run {
         $realSession->{$sphg} ||= {};
 
         # Merge specified groups/hGroups only
+        my %intersct = %{ $realSession->{$sphg} };
         unless ( $self->{conf}->{impersonationMergeSSOgroups} eq 1 ) {
+            %intersct = ();
             my %SSOgroups = map { $_, 1 } split /\Q$separator/,
               $self->{conf}->{impersonationMergeSSOgroups};
 
             $self->logger->debug("Filtering specified groups/hGroups...");
             @realGrps = grep { exists $SSOgroups{$_} } @realGrps;
-            my %intersct =
+            %intersct =
               map {
                 $realSession->{$sphg}->{$_}
                   ? ( $_, $realSession->{$sphg}->{$_} )
                   : ()
               } keys %SSOgroups;
-            $realSession->{$sphg} = \%intersct;
         }
 
         $self->logger->debug("Processing groups...");
@@ -148,7 +182,7 @@ sub run {
 
         $self->logger->debug("Processing hGroups...");
         $spoofSession->{hGroups} =
-          { %{ $spoofSession->{hGroups} }, %{ $realSession->{$sphg} } };
+          { %{ $spoofSession->{hGroups} }, %intersct };
     }
 
     # Main session
@@ -168,10 +202,10 @@ sub run {
 }
 
 sub _userData {
-    my ( $self, $req, $spoofId, $realSession ) = @_;
+    my ( $self, $req, $spoofId, $realSession, $unUser ) = @_;
     my $realId = $req->{user};
+    my $raz    = 0;
     $req->{user} = $spoofId;
-    my $raz = 0;
 
     # Compute Macros and Groups with real and spoof sessions
     $req->sessionInfo($realSession);
@@ -184,7 +218,7 @@ sub _userData {
     if ( my $error = $self->p->process($req) ) {
         if ( $error == PE_BADCREDENTIALS ) {
             $self->userLogger->warn(
-                    'Impersonation requested for an unvalid user ('
+                    'Impersonation requested for an invalid user ('
                   . $req->{user}
                   . ")" );
         }
@@ -193,11 +227,12 @@ sub _userData {
         $raz = 1;
     }
 
-    # Check identity rule if Impersonation required
+    # Check identities rule if Impersonation required
     if ( $realId ne $spoofId ) {
-        unless ( $self->idRule->( $req, $req->sessionInfo ) ) {
+        $self->logger->info("\"$realId\" is an unrestricted user!") if $unUser;
+        unless ( $unUser || $self->idRule->( $req, $req->sessionInfo ) ) {
             $self->userLogger->warn(
-                    'Impersonation requested for an unvalid user ('
+                    'Impersonation requested for an invalid user ('
                   . $req->{user}
                   . ")" );
             $self->logger->debug('Identity NOT authorized');
@@ -215,7 +250,7 @@ sub _userData {
                 $self->p->groupsAndMacros, 'setLocalGroups'
             ]
         );
-        $self->logger->debug('Spoof session equal real session');
+        $self->logger->debug('Reset Impersonation process');
         $req->error(PE_BADCREDENTIALS);
         if ( my $error = $self->p->process($req) ) {
             $self->logger->debug("Process returned error: $error");
@@ -238,4 +273,14 @@ sub _userData {
     return $req->{sessionInfo};
 }
 
+sub setSecurity {
+    my ( $self, $req ) = @_;
+    if ( $self->captcha ) {
+        $self->captcha->setCaptcha($req);
+    }
+    elsif ( $self->ottRule->( $req, {} ) ) {
+        $self->ott->setToken($req);
+    }
+}
+
 1;
