@@ -167,7 +167,8 @@ sub myNotifs {
     my ( $self, $req, $ref ) = @_;
 
     if ($ref) {
-        return $self->sendJSONresponse( $req, { error => 'Missing epoch parameter' } )
+        return $self->sendJSONresponse( $req,
+            { error => 'Missing epoch parameter' } )
           unless $req->param('epoch');
 
         # Retrieve notification reference=$ref with epoch
@@ -212,14 +213,14 @@ sub retrieveNotifs {
     my @_notifications = sort {
              $b->{epoch} <=> $a->{epoch}
           or $a->{reference} cmp $b->{reference}
-      } (
+    } (
         map {
             /^notification_(.+)$/
               ? { reference => $1, epoch => $req->{userData}->{$_} }
               : ()
           }
           keys %{ $req->{userData} }
-      );
+    );
     splice @_notifications, $self->conf->{notificationsMaxRetrieve};
 
     return \@_notifications;
@@ -228,10 +229,11 @@ sub retrieveNotifs {
 sub _viewNotif {
     my ( $self, $req, $ref, $epoch ) = @_;
 
-    $self->logger->debug( "Retrieve notification with reference: \"$ref\" and epoch: \"$epoch\"" );
+    $self->logger->debug(
+        "Retrieve notification with reference: \"$ref\" and epoch: \"$epoch\"");
     my $notif = eval { $self->module->viewNotification( $req, $ref, $epoch ); };
     if ($@) {
-    	$self->logger->debug( "Notification not found" );
+        $self->logger->debug("Notification not found");
         $self->logger->error($@);
         return '';
     }
