@@ -55,7 +55,9 @@ C<LLNG::Manager::Test::_post()> call I<(see below)>.
 
 use strict;
 use Data::Dumper;
+use File::Find;
 use LWP::UserAgent;
+use Time::Fake;
 use URI::Escape;
 use Lemonldap::NG::Common::FormEncode;
 
@@ -73,6 +75,15 @@ $Data::Dumper::Sortkeys = 1;
 $Data::Dumper::Useperl  = 1;
 my $ini;
 
+use File::Temp 'tempfile', 'tempdir';
+use File::Copy 'copy';
+our $tmpDir = $LLNG::TMPDIR
+  || tempdir( 'tmpSessionXXXXX', DIR => 't/sessions', CLEANUP => 1 );
+mkdir "$tmpDir/lock";
+mkdir "$tmpDir/saml";
+mkdir "$tmpDir/saml/lock";
+copy( "t/lmConf-1.json", "$tmpDir/lmConf-1.json" );
+
 =head4 count($inc)
 
 Returns number of tests done. Increment test number if an argument is given
@@ -85,6 +96,29 @@ sub count {
     return $count;
 }
 
+=head4 buildForm($params)
+
+Convenience method that builds a url-encoded query string from a hash of arguments
+
+=cut
+
+sub buildForm {
+    my $fields = shift;
+    my $query  = join(
+        '&',
+        map {
+            "$_="
+              . (
+                $fields->{$_}
+                ? uri_escape( uri_unescape( $fields->{$_} ) )
+                : ''
+              )
+          }
+          keys(%$fields)
+    );
+    return $query;
+}
+
 =head4 explain( $result, $expected_result )
 
 Used to display error if test fails:
@@ -107,10 +141,7 @@ Clean sessions created during tests
 =cut
 
 sub clean_sessions {
-    opendir D, 't/sessions' or die $!;
-    foreach ( grep { /^[^\.]/ } readdir(D) ) {
-        unlink "t/sessions/$_", "t/sessions/lock/Apache-Session-$_.lock";
-    }
+    find( sub { unlink if -f }, $tmpDir );
     foreach my $dir (qw(t/sessions/lock t/sessions/saml/lock t/sessions/saml)) {
         if ( -d $dir ) {
             opendir D, $dir or die $!;
@@ -119,15 +150,34 @@ sub clean_sessions {
             }
         }
     }
-    my $cache = getCache();
-    $cache->clear;
+}
+
+sub count_sessions {
+    my ( $kind, $dir ) = @_;
+    my $nbr = 0;
+    $kind ||= 'SSO';
+    $dir  ||= $tmpDir;
+
+    opendir D, $dir or die $!;
+    foreach ( grep { /^\w{64}$/ } readdir(D) ) {
+        open( my $fh, '<', "$dir/$_" ) or die($!);
+        while (<$fh>) {
+            chomp;
+            if ( $_ =~ /"_session_kind":"$kind"/ ) {
+                $nbr++;
+                last;
+            }
+        }
+        close $fh;
+    }
+    $nbr;
 }
 
 sub getCache {
     require Cache::FileCache;
     return Cache::FileCache->new( {
             namespace   => 'lemonldap-ng-session',
-            cache_root  => 't/',
+            cache_root  => $tmpDir,
             cache_depth => 0,
         }
     );
@@ -258,18 +308,7 @@ m@<form.+?action="(?:(?:http://([^/]+))?(/.*?)?|(#))".+method="(post|get)"@is,
               m#<input.+?name="([^"]+)"[^>]+(?:value="([^"]*?)")?#gs,
             %fields
         );
-        my $query = join(
-            '&',
-            map {
-                "$_="
-                  . (
-                    $fields{$_}
-                    ? uri_escape( uri_unescape( $fields{$_} ) )
-                    : ''
-                  )
-              }
-              keys(%fields)
-        );
+        my $query = buildForm( \%fields );
         foreach my $f (@requiredFields) {
             ok( exists $fields{$f}, qq{ Field "$f" is defined} );
             count(1);
@@ -308,6 +347,26 @@ sub expectOK {
     count(1);
 }
 
+=head4 expectJSON($res)
+
+Verify that the HTTP response contains valid JSON and returns the corresponding object
+
+=cut
+
+sub expectJSON {
+    my ($res) = @_;
+    is( $res->[0], 200, ' HTTP code is 200' ) or explain( $res, 200 );
+    my %hdr = @{ $res->[1] };
+    like( $hdr{'Content-Type'}, qr,^application/json,i,
+        ' Content-Type is JSON' )
+      or explain($res);
+    my $json;
+    eval { $json = JSON::from_json( $res->[2]->[0] ) };
+    ok( not($@), 'Content is valid JSON' );
+    count(3);
+    return $json;
+}
+
 =head4 expectBadRequest($res)
 
 Verify that returned code is 400. Note that it works only for Ajax request
@@ -321,6 +380,18 @@ sub expectBadRequest {
     count(1);
 }
 
+=head4 expectPortalError( $res, $errnum )
+
+Verify that an error is displayed on the portal
+=cut
+
+sub expectPortalError {
+    my ( $res, $errnum, $message ) = @_;
+    $errnum ||= 9;
+    like( $res->[2]->[0], qr/<span trmsg="$errnum">/, $message );
+    count(1);
+}
+
 =head4 expectReject( $res, $code )
 
 Verify that returned code is 401 and JSON result contains C<error:"$code">.
@@ -329,14 +400,14 @@ Note that it works only for Ajax request (see below).
 =cut
 
 sub expectReject {
-    my ( $res, $code ) = @_;
-    ok( $res->[0] == 401, ' Response is 401' ) or explain( $res->[0], 401 );
+    my ( $res, $status, $code ) = @_;
+    $status ||= 401;
+    cmp_ok( $res->[0], '==', $status, " Response status is $status" );
     eval { $res = JSON::from_json( $res->[2]->[0] ) };
-    ok( not($@), 'Content is JSON' )
+    ok( not($@), ' Content is JSON' )
       or explain( $res->[2]->[0], 'JSON content' );
     if ( defined $code ) {
-        ok( $res->{error} == $code, "Error code is $code" )
-          or explain( $res->{error}, $code );
+        is( $res->{error}, $code, " Error code is $code" );
     }
     else {
         pass("Error code is $res->{error}");
@@ -380,7 +451,8 @@ sub exceptCspFormOK {
     }
     if (   $csp =~ /\s\*(?:\s.*)?\s*$/
         or ( $host eq '#' and $csp =~ /'self'/ )
-        or $csp =~ m#\bhttps?://$host\b# )
+        or $csp =~ m#\bhttps?://$host\b#
+        or $csp =~ m#\*# )
     {
         pass(" CSP header authorize POST request to $host");
     }
@@ -453,6 +525,50 @@ sub getUser {
     return getHeader( $resp, 'Lm-Remote-User' );
 }
 
+=head4 tempdb
+
+Return a temporary file named XXXX.db
+
+=cut
+
+sub tempdb {
+    return "$tmpDir/userdb.db";
+}
+
+my %handlerOR;
+
+=head4 register
+
+Registers a new LLNG instance
+
+=cut
+
+sub register {
+    my ( $type, $constructor ) = @_;
+    my $obj;
+    @Lemonldap::NG::Handler::Main::_onReload = ();
+    &Lemonldap::NG::Handler::Main::cfgNum( 0, 0 );
+    ok( $obj = $constructor->(), 'Register $type' );
+    count(1);
+    $handlerOR{$type} = \@Lemonldap::NG::Handler::Main::_onReload;
+    return $obj;
+}
+
+=head4 register
+
+Switch to a registered instance
+
+=cut
+
+sub switch {
+    my $type = shift;
+    return [] unless $handlerOR{$type};
+    note( '==> Switching to ' . uc($type) . ' <==' );
+    @Lemonldap::NG::Handler::Main::_onReload = @{
+        $handlerOR{$type};
+    };
+}
+
 =head2 LLNG::Manager::Test Class
 
 =cut
@@ -467,21 +583,46 @@ extends 'Lemonldap::NG::Common::PSGI::Cli::Lib';
 our $defaultIni = {
     configStorage => {
         type    => 'File',
-        dirName => 't',
+        dirName => "$tmpDir",
     },
     localSessionStorage        => 'Cache::FileCache',
     localSessionStorageOptions => {
         namespace   => 'lemonldap-ng-session',
-        cache_root  => 't/',
+        cache_root  => $tmpDir,
         cache_depth => 0,
     },
-    logLevel      => 'error',
-    cookieName    => 'lemonldap',
-    domain        => 'example.com',
-    templateDir   => 'site/templates',
-    staticPrefix  => '/static',
-    securedCookie => 0,
-    https         => 0,
+    logLevel              => 'error',
+    cookieName            => 'lemonldap',
+    domain                => 'example.com',
+    templateDir           => 'site/templates',
+    staticPrefix          => '/static',
+    tokenUseGlobalStorage => 0,
+    securedCookie         => 0,
+    https                 => 0,
+    globalStorageOptions  => {
+        Directory     => $tmpDir,
+        LockDirectory => "$tmpDir/lock",
+        generateModule =>
+          'Lemonldap::NG::Common::Apache::Session::Generate::SHA256',
+    },
+    casStorageOptions => {
+        Directory     => "$tmpDir/saml",
+        LockDirectory => "$tmpDir/saml/lock",
+        generateModule =>
+          'Lemonldap::NG::Common::Apache::Session::Generate::SHA256',
+    },
+    samlStorageOptions => {
+        Directory     => "$tmpDir/saml",
+        LockDirectory => "$tmpDir/saml/lock",
+        generateModule =>
+          'Lemonldap::NG::Common::Apache::Session::Generate::SHA256',
+    },
+    oidcStorageOptions => {
+        Directory     => "$tmpDir/saml",
+        LockDirectory => "$tmpDir/saml/lock",
+        generateModule =>
+          'Lemonldap::NG::Common::Apache::Session::Generate::SHA256',
+    },
 };
 
 =head3 Accessors
@@ -513,6 +654,8 @@ has p => ( is => 'rw' );
 
 =cut
 
+has confFailure => ( is => 'rw' );
+
 has ini => (
     is      => 'rw',
     lazy    => 1,
@@ -524,22 +667,30 @@ has ini => (
         }
         $self->{ini} = $ini;
         main::ok( $self->{p} = $self->class->new(), 'Portal object' );
-        main::ok( $self->{p}->init($ini), 'Init' );
-        main::ok( $self->{app} = $self->{p}->run(), 'Portal app' );
-        main::count(3);
-        no warnings 'redefine';
-        eval
+        main::count(1);
+        unless ( $self->confFailure ) {
+            main::ok( $self->{p}->init($ini),           'Init' );
+            main::ok( $self->{app} = $self->{p}->run(), 'Portal app' );
+            main::count(2);
+            no warnings 'redefine';
+            eval
 'sub Lemonldap::NG::Common::Logger::Std::error {return $_[0]->warn($_[1])}';
-        $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{french} = {
-            uid  => 'french',
-            cn   => 'Frédéric Accents',
-            mail => 'fa@badwolf.org',
-        };
-        $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{russian} = {
-            uid  => 'russian',
-            cn   => 'Русский',
-            mail => 'ru@badwolf.org',
-        };
+            $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{french} = {
+                uid  => 'french',
+                cn   => 'Frédéric Accents',
+                mail => 'fa@badwolf.org',
+            };
+            $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{davros} = {
+                uid  => 'davros',
+                cn   => 'Bad Guy',
+                mail => 'davros@badguy.org',
+            };
+            $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{russian} = {
+                uid  => 'russian',
+                cn   => 'Русский',
+                mail => 'ru@badwolf.org',
+            };
+        }
         $self;
     }
 );
@@ -629,7 +780,7 @@ sub _get {
             'HTTP_ACCEPT_LANGUAGE' => 'fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3',
             'HTTP_CACHE_CONTROL'   => 'max-age=0',
             ( $args{cookie} ? ( HTTP_COOKIE => $args{cookie} ) : () ),
-            'HTTP_HOST' => 'auth.example.com',
+            'HTTP_HOST' => ( $args{host} ? $args{host} : 'auth.example.com' ),
             'HTTP_USER_AGENT' =>
               'Mozilla/5.0 (VAX-4000; rv:36.0) Gecko/20350101 Firefox',
             'PATH_INFO' => $path,
@@ -681,16 +832,18 @@ sub _post {
             'HTTP_ACCEPT_LANGUAGE' => 'fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3',
             'HTTP_CACHE_CONTROL'   => 'max-age=0',
             ( $args{cookie} ? ( HTTP_COOKIE => $args{cookie} ) : () ),
-            'HTTP_HOST' => 'auth.example.com',
+            'HTTP_HOST' => ( $args{host} ? $args{host} : 'auth.example.com' ),
             'HTTP_USER_AGENT' =>
               'Mozilla/5.0 (VAX-4000; rv:36.0) Gecko/20350101 Firefox',
             'PATH_INFO' => $path,
             ( $args{query}   ? ( QUERY_STRING => $args{query} )   : () ),
             ( $args{referer} ? ( REFERER      => $args{referer} ) : () ),
-            'REMOTE_ADDR' => '127.0.0.1',
             (
-                $args{remote_user}
-                ? ( 'REMOTE_USER' => $args{remote_user} )
+                $args{ip} ? ( 'REMOTE_ADDR' => $args{ip} )
+                : ( 'REMOTE_ADDR' => '127.0.0.1' )
+            ),
+            (
+                $args{remote_user} ? ( 'REMOTE_USER' => $args{remote_user} )
                 : ()
             ),
             'REQUEST_METHOD' => $args{method} || 'POST',
@@ -719,7 +872,19 @@ Call C<_get()> with method set to DELETE.
 sub _delete {
     my ( $self, $path, %args ) = @_;
     $args{method} = 'DELETE';
-    return $self->_get( $path, %args );
+    $self->_get( $path, %args );
+}
+
+=head4 _options( $path, %args )
+
+Call C<_get()> with method set to OPTIONS.
+
+=cut
+
+sub _options {
+    my ( $self, $path, %args ) = @_;
+    $args{method} = 'OPTIONS';
+    $self->_get( $path, %args );
 }
 
 =head4 _put( $path, $body, %args )
